#
#Copyright (c) 2008, Institute of Computer Technology (University of Technology Vienna)
#Authors:
#        Joseph Wenninger <wenninger@ict.tuwien.ac.at>
#
# Modules for the testbench for the ANDRES converter channel
# 0.0  - 09.09.2008
# (c) Joseph Wenninger
# Institute of Computer Technology
# Vienna University of Technology

IF (MSVC)
  MESSAGE(STATUS "!!!!!!!!!!!! MSVC !!!!!!!!!!!!")
  SET(CMAKE_CXX_FLAGS_DEBUG "${CMAKE_CXX_FLAGS_DEBUG} /MTd /GR /vmg")
  SET(CMAKE_CXX_FLAGS_RELEASE "${CMAKE_CXX_FLAGS_RELEASE} /MT /GR /vmg")
ELSE (MSVC)
  SET(CMAKE_BUILD_TYPE "Debug" CACHE STRING  "build type determining compiler flags" FORCE )
ENDIF (MSVC)

MESSAGE(STATUS "CMake believes system processor is: ${CMAKE_SYSTEM_PROCESSOR}")
MESSAGE(STATUS "CMake believes system name is: ${CMAKE_SYSTEM_NAME}")
MESSAGE(STATUS "****TODO: Switch SystemC arch detection over to cmake's internal variables")
MESSAGE(STATUS "****Ignoring CMake believes, asking gcc for it's interpretation")

SET(SYSTEMC_INCLUDE_SEARCH_PATH "~/systemc/include" "~/systemc/systemc/include" "/usr/include")
SET(SYSTEMC_LIBRARY_SEARCH_PATH "~/systemc" "~/systemc/systemc" "/usr/lib")

SET(SYSTEMC_HOME "$ENV{SYSTEMC_HOME}")
IF(SYSTEMC_HOME)
  SET(SYSTEMC_INCLUDE_SEARCH_PATH "${SYSTEMC_HOME}/include" ${SYSTEMC_INCLUDE_SEARCH_PATH})
  #SET(SYSTEMC_LIBRARY_SEARCH_PATH "${SYSTEMC_HOME}" ${SYSTEMC_LIBRARY_SEARCH_PATH})
  SET(SYSTEMC_LIBRARY_SEARCH_PATH "${SYSTEMC_HOME}/SystemC/Release" "${SYSTEMC_HOME}" ${SYSTEMC_LIBRARY_SEARCH_PATH})
  SET(SYSTEMC_LIBRARY_SEARCH_PATH_DEBUG "${SYSTEMC_HOME}/SystemC/Debug" "${SYSTEMC_HOME}" ${SYSTEMC_LIBRARY_SEARCH_PATH})
  IF (MSVC)
    SET(SYSTEMC_INCLUDE_SEARCH_PATH "${SYSTEMC_HOME}/../src" ${SYSTEMC_INCLUDE_SEARCH_PATH})
  ENDIF (MSVC)
ENDIF(SYSTEMC_HOME)

SET(SYSTEMC_INCLUDE_PATH "$ENV{SYSTEMC_INCLUDE_PATH}")
IF(SYSTEMC_INCLUDE_PATH)
  SET(SYSTEMC_INCLUDE_SEARCH_PATH "${SYSTEMC_INCLUDE_PATH}" ${SYSTEMC_INCLUDE_SEARCH_PATH})
ENDIF(SYSTEMC_INCLUDE_PATH)

FIND_PATH(SYSTEMC_INCLUDES systemc.h ${SYSTEMC_INCLUDE_SEARCH_PATH})
IF(NOT SYSTEMC_INCLUDES)
  MESSAGE(FATAL_ERROR "systemc.h could not be found, pathes searched are ${SYSTEMC_INCLUDE_SEARCH_PATH}\n--------->Try setting SYSTEMC_HOME to the root of your SYSTEMC installation, if you don't use the standard unix/windows directory layout, you could set SYSTEMC_INCLUDE_PATH to the path of the includes")
ELSE(NOT SYSTEMC_INCLUDES)
  MESSAGE(STATUS "SYSTEMC includes are in ${SYSTEMC_INCLUDES}")
ENDIF(NOT SYSTEMC_INCLUDES)



IF(SYSTEMC_LIBRARIES)
  MESSAGE(STATUS "SYSTEMC_LIBRARIES is already set, no autodetection is performed")
  IF(NOT EXISTS "${SYSTEMC_LIBRARIES}")
    MESSAGE(FATAL_ERROR "The CMake variable SYSTEMC_LIBRARIES points to an non existent file, if you have moved your SystemC installation remove the CMake cache and rerun cmake, otherwise fix your manually added SET line in your CMakeLists.txt")
  ENDIF(NOT EXISTS "${SYSTEMC_LIBRARIES}")
ELSE(SYSTEMC_LIBRARIES)
  IF (MSVC)
    FIND_LIBRARY(SYSTEMC_LIB systemc PATHS ${SYSTEMC_LIBRARY_SEARCH_PATH})
    FIND_LIBRARY(SYSTEMC_LIB_DEBUG systemc PATHS ${SYSTEMC_LIBRARY_SEARCH_PATH_DEBUG})
    SET(SYSTEMC_LIBRARIES  optimized ${SYSTEMC_LIB} debug ${SYSTEMC_LIB_DEBUG} CACHE FILEPATH "SYSTEMC_LIBRARIES")
  ELSE (MSVC)
    IF(NOT CMAKE_COMPILER_IS_GNUCXX)
      MESSAGE(FATAL_ERROR "The used compiler \${CMAKE_CXX_COMPILER}==${CMAKE_CXX_COMPILER} is not a GNU C++\n-----> try setting CXX to a valid GNU C++ compiler")
    ENDIF(NOT CMAKE_COMPILER_IS_GNUCXX)
    EXECUTE_PROCESS(COMMAND ${CMAKE_CXX_COMPILER} -dumpmachine
            OUTPUT_VARIABLE SYSTEMC_CHECK_CXX_ARCH ERROR_VARIABLE SYSTEMC_CHECK_CXX_ERROR)
    IF(SYSTEMC_CHECK_CXX_ERROR)
            MESSAGE(FATAL_ERROR "The used compiler \${CMAKE_CXX_COMPILER}==${CMAKE_CXX_COMPILER} doesn't understand -dumpmachine (no GNU c++ ?)\n-----> try setting CXX to a valid GNU C++ compiler")
    ENDIF(SYSTEMC_CHECK_CXX_ERROR)
    
    MESSAGE(STATUS "Compiler Arch string: ${SYSTEMC_CHECK_CXX_ARCH}")
    MESSAGE(STATUS "Compiler is: ${CMAKE_CXX_COMPILER}")
  
    SET(SYSTEMC_OVERRIDE_ARCH "$ENV{SYSTEMC_OVERRIDE_ARCH}")
    IF (SYSTEMC_OVERRIDE_ARCH)
      MESSAGE(STATUS "Overriding arch with ${SYSTEMC_OVERRIDE_ARCH}")
      SET(SYSTEMC_CHECK_CXX_ARCH "${SYSTEMC_OVERRIDE_ARCH}")
    ENDIF (SYSTEMC_OVERRIDE_ARCH)
    
    SET(SYSTEMC_BIT "HALLO")
  
    IF(SYSTEMC_CHECK_CXX_ARCH MATCHES "sparc-sun-solaris.*")
      SET(SYSTEMC_ARCH_DIR_SUFFIX "gccsparcOS5")
      SET(SYSTEMC_BIT "32")
    ELSEIF(SYSTEMC_CHECK_CXX_ARCH MATCHES "x86_64.*linux.*")
      SET(SYSTEMC_ARCH_DIR_SUFFIX "linux64")
      SET(SYSTEMC_BIT "64")
    ELSEIF(SYSTEMC_CHECK_CXX_ARCH MATCHES ".*linux.*")
      SET(SYSTEMC_ARCH_DIR_SUFFIX "linux")
      SET(SYSTEMC_BIT "32")
    ELSEIF(SYSTEMC_CHECK_CXX_ARCH MATCHES ".*-cygwin")
      SET(SYSTEMC_ARCH_DIR_SUFFIX "cygwin")
      SET(SYSTEMC_BIT "32")
    ELSEIF(SYSTEMC_CHECK_CXX_ARCH MATCHES "i686-apple-darwin9")
      SET(SYSTEMC_ARCH_DIR_SUFFIX "macosx")
      SET(SYSTEMC_BIT "32")
    ELSE(SYSTEMC_CHECK_CXX_ARCH MATCHES "sparc-sun-solaris.*")
      MESSAGE(FATAL_ERROR "SystemC architecture detection not supported for this arch.\n"
        "Please set SYSTEMC_LIBRARIES in your CMakeLists.txt before FIND_PACKAGE(SystemC)")
    ENDIF(SYSTEMC_CHECK_CXX_ARCH MATCHES "sparc-sun-solaris.*")
  
    FIND_LIBRARY(SYSTEMC_LIB systemc PATHS ${SYSTEMC_LIBRARY_SEARCH_PATH} PATH_SUFFIXES "lib-${SYSTEMC_ARCH_DIR_SUFFIX}")
    IF(NOT SYSTEMC_LIB)
      MESSAGE(FATAL_ERROR "SystemC library <systemc> could not be found, pathes searched are ${SYSTEMC_LIBRARY_SEARCH_PATH}, suffix is <lib-${SYSTEMC_ARCH_DIR_SUFFIX}>\n--------->Try setting SYSTEMC_HOME to the root of your SystemC installation")
    ENDIF(NOT SYSTEMC_LIB)
    SET(SYSTEMC_LIBRARIES optimized ${SYSTEMC_LIB} debug ${SYSTEMC_LIB} CACHE FILEPATH  "SYSTEMC_LIBRARIES")
  ENDIF(MSVC)

#MESSAGE(STATUS ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> ${SYSTEMC_BIT}")
ENDIF(SYSTEMC_LIBRARIES)
#MESSAGE(STATUS "2>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> ${SYSTEMC_BIT}")
IF (SYSTEMC_BIT)
	SET(SYSTEMC_BIT ${SYSTEMC_BIT} CACHE STRING "32 or 64 bit lib")
ENDIF(SYSTEMC_BIT)


INCLUDE_DIRECTORIES(${SYSTEMC_INCLUDES})
MESSAGE(STATUS "SYSTEMC_LIBRARIES: ${SYSTEMC_LIBRARIES}")
# kate: space-indent on; indent-width 2; replace-tabs on;
