#
#Copyright (c) 2008, Institute of Computer Technology (University of Technology Vienna)
#Authors:
#        Joseph Wenninger <wenninger@ict.tuwien.ac.at>
#
# Modules for the testbench for the ANDRES converter channel
# 0.0  - 09.09.2008
# (c) Joseph Wenninger
# Institute of Computer Technology
# Vienna University of Technology


SET(CMAKE_BUILD_TYPE "Debug" CACHE STRING  "build type determining compiler flags" FORCE )


MESSAGE(STATUS "CMake believes system processor is: ${CMAKE_SYSTEM_PROCESSOR}")
MESSAGE(STATUS "CMake believes system name is: ${CMAKE_SYSTEM_NAME}")
MESSAGE(STATUS "****TODO: Switch SystemC arch detection over to cmake's internal variables")
MESSAGE(STATUS "****Ignoring CMake believes, asking gcc for it's interpretation")

SET(SYSTEMCAMS_INCLUDE_SEARCH_PATH "~/systemc-ams/include" "~/systemc-ams/systemc-ams/include" "/usr/include")
SET(SYSTEMCAMS_LIBRARY_SEARCH_PATH "~/systemc-ams" "~/systemc-ams/systemc-ams" "/usr/lib")

SET(SYSTEMCAMS_HOME "$ENV{SYSTEMCAMS_HOME}")
IF(SYSTEMCAMS_HOME)
  SET(SYSTEMCAMS_INCLUDE_SEARCH_PATH "${SYSTEMCAMS_HOME}/include" ${SYSTEMCAMS_INCLUDE_SEARCH_PATH})
  #SET(SYSTEMCAMS_LIBRARY_SEARCH_PATH "${SYSTEMCAMS_HOME}" ${SYSTEMCAMS_LIBRARY_SEARCH_PATH})
  SET(SYSTEMCAMS_LIBRARY_SEARCH_PATH "${SYSTEMCAMS_HOME}/systemc-ams/Release" "${SYSTEMCAMS_HOME}/Release" "${SYSTEMCAMS_HOME}" ${SYSTEMCAMS_LIBRARY_SEARCH_PATH})
  SET(SYSTEMCAMS_LIBRARY_SEARCH_PATH_DEBUG "${SYSTEMCAMS_HOME}/systemc-ams/Debug" "${SYSTEMCAMS_HOME}/Debug" "${SYSTEMCAMS_HOME}" ${SYSTEMCAMS_LIBRARY_SEARCH_PATH})
  IF (MSVC)
    SET(SYSTEMCAMS_INCLUDE_SEARCH_PATH "${SYSTEMCAMS_HOME}/../src" ${SYSTEMCAMS_INCLUDE_SEARCH_PATH})
  ENDIF (MSVC)
ENDIF(SYSTEMCAMS_HOME)


SET(SYSTEMCAMS_INCLUDE_PATH "$ENV{SYSTEMCAMS_INCLUDE_PATH}")
IF(SYSTEMCAMS_INCLUDE_PATH)
  SET(SYSTEMCAMS_INCLUDE_SEARCH_PATH "${SYSTEMCAMS_INCLUDE_PATH}" ${SYSTEMC_INCLUDE_SEARCH_PATH})
ENDIF(SYSTEMCAMS_INCLUDE_PATH)

#MESSAGE(STATUS "SYSTEMCAMS_LIBRARY_SEARCH_PATH:${SYSTEMCAMS_LIBRARY_SEARCH_PATH}")
#MESSAGE(STATUS "SYSTEMCAMS_LIBRARY_SEARCH_PATH_DEBUG:${SYSTEMCAMS_LIBRARY_SEARCH_PATH_DEBUG}")

FIND_PATH(SYSTEMCAMS_INCLUDES systemc-ams.h ${SYSTEMCAMS_INCLUDE_SEARCH_PATH})
IF(NOT SYSTEMCAMS_INCLUDES)
  MESSAGE(FATAL_ERROR "systemc-ams.h could not be found, pathes searched are ${SYSTEMCAMS_INCLUDE_SEARCH_PATH}\n--------->Try setting SYSTEMCAMS_HOME to the root of your SystemC-AMS installation  installation, if you don't use the standard unix/windows directory layout, you could set SYSTEMCAMS_INCLUDE_PATH to the path of the includes")
ELSE(NOT SYSTEMCAMS_INCLUDES)
  MESSAGE(STATUS "SYSTEMCAMS includes are in ${SYSTEMCAMS_INCLUDES}")
ENDIF(NOT SYSTEMCAMS_INCLUDES)



IF(SYSTEMCAMS_LIBRARIES)
  MESSAGE(STATUS "SYSTEMCAMS_LIBRARIES is already set, no autodetection is performed")
  IF (NOT SYSTEMC_DONT_CHECK_EXISTING)
    IF(NOT EXISTS "${SYSTEMCAMS_LIBRARIES}")
      MESSAGE(FATAL_ERROR "The CMake variable SYSTEMCAMS_LIBRARIES points to an non existent file, if you have moved your SystemC installation remove the CMake cache and rerun cmake, otherwise fix your manually added SET line in your CMakeLists.txt")
    ENDIF(NOT EXISTS "${SYSTEMCAMS_LIBRARIES}")
  ENDIF (NOT SYSTEMC_DONT_CHECK_EXISTING)
ELSE(SYSTEMCAMS_LIBRARIES)
  IF (MSVC)
    FIND_LIBRARY(SYSTEMCAMS_LIB systemc_ams PATHS ${SYSTEMCAMS_LIBRARY_SEARCH_PATH})
    FIND_LIBRARY(SYSTEMCAMS_LIB_DEBUG systemc_amsd PATHS ${SYSTEMCAMS_LIBRARY_SEARCH_PATH_DEBUG})
    IF (NOT SYSTEMCAMS_LIB_DEBUG)
      FIND_LIBRARY(SYSTEMCAMS_LIB_DEBUG systemc_ams PATHS ${SYSTEMCAMS_LIBRARY_SEARCH_PATH_DEBUG})
    ENDIF (NOT SYSTEMCAMS_LIB_DEBUG)
    SET(SYSTEMCAMS_LIBRARIES optimized ${SYSTEMCAMS_LIB} debug ${SYSTEMCAMS_LIB_DEBUG} CACHE FILEPATH "SYSTEMCAMS_LIBRARIES")
  ELSE (MSVC)
    IF(NOT CMAKE_COMPILER_IS_GNUCXX)
      MESSAGE(FATAL_ERROR "The used compiler \${CMAKE_CXX_COMPILER}==${CMAKE_CXX_COMPILER} is not a GNU C++\n-----> try setting CXX to a valid GNU C++ compiler")
    ENDIF(NOT CMAKE_COMPILER_IS_GNUCXX)
    EXECUTE_PROCESS(COMMAND ${CMAKE_CXX_COMPILER} -dumpmachine
            OUTPUT_VARIABLE SYSTEMCAMS_CHECK_CXX_ARCH ERROR_VARIABLE SYSTEMCAMS_CHECK_CXX_ERROR)
    IF(SYSTEMCAMS_CHECK_CXX_ERROR)
            MESSAGE(FATAL_ERROR "The used compiler \${CMAKE_CXX_COMPILER}==${CMAKE_CXX_COMPILER} doesn't understand -dumpmachine (no GNU c++ ?)\n-----> try setting CXX to a valid GNU C++ compiler")
    ENDIF(SYSTEMCAMS_CHECK_CXX_ERROR)
    
    MESSAGE(STATUS "Compiler Arch string: ${SYSTEMCAMS_CHECK_CXX_ARCH}")
    MESSAGE(STATUS "Compiler is: ${CMAKE_CXX_COMPILER}")
  
    SET(SYSTEMCAMS_OVERRIDE_ARCH "$ENV{SYSTEMCAMS_OVERRIDE_ARCH}")
    IF (SYSTEMCAMS_OVERRIDE_ARCH)
      MESSAGE(STATUS "Overriding arch with ${SYSTEMCAMS_OVERRIDE_ARCH}")
      SET(SYSTEMCAMS_CHECK_CXX_ARCH "${SYSTEMCAMS_OVERRIDE_ARCH}")
    ENDIF (SYSTEMCAMS_OVERRIDE_ARCH)
    
  
    IF(SYSTEMCAMS_CHECK_CXX_ARCH MATCHES "sparc-sun-solaris.*")
      SET(SYSTEMCAMS_ARCH_DIR_SUFFIX "gccsparcOS5")
      SET(SYSTEMCAMS_BIT "32")
    ELSEIF(SYSTEMCAMS_CHECK_CXX_ARCH MATCHES "x86_64.*linux.*")
      SET(SYSTEMCAMS_ARCH_DIR_SUFFIX "linux64")
      SET(SYSTEMCAMS_BIT "64")
    ELSEIF(SYSTEMCAMS_CHECK_CXX_ARCH MATCHES ".*linux.*")
      SET(SYSTEMCAMS_ARCH_DIR_SUFFIX "linux")
      SET(SYSTEMCAMS_BIT "32")
    ELSEIF(SYSTEMCAMS_CHECK_CXX_ARCH MATCHES ".*-cygwin")
      SET(SYSTEMCAMS_ARCH_DIR_SUFFIX "cygwin")
      SET(SYSTEMCAMS_BIT "32")
    ELSEIF(SYSTEMCAMS_CHECK_CXX_ARCH MATCHES "i686-apple-darwin9")
      SET(SYSTEMCAMS_ARCH_DIR_SUFFIX "macosx")
      SET(SYSTEMCAMS_BIT "32")
    ELSE(SYSTEMCAMS_CHECK_CXX_ARCH MATCHES "sparc-sun-solaris.*")
      MESSAGE(FATAL_ERROR "SystemC architecture detection not supported for this arch.\n"
        "Please set SYSTEMCAMS_LIBRARIES in your CMakeLists.txt before FIND_PACKAGE(SystemC)")
    ENDIF(SYSTEMCAMS_CHECK_CXX_ARCH MATCHES "sparc-sun-solaris.*")
  
    FIND_LIBRARY(SYSTEMCAMS_LIB systemc-ams PATHS ${SYSTEMCAMS_LIBRARY_SEARCH_PATH} PATH_SUFFIXES "lib")
    IF(NOT SYSTEMCAMS_LIB)
      FIND_LIBRARY(SYSTEMCAMS_LIB systemc-ams PATHS ${SYSTEMCAMS_LIBRARY_SEARCH_PATH} PATH_SUFFIXES "lib-${SYSTEMCAMS_ARCH_DIR_SUFFIX}")
      IF(NOT SYSTEMCAMS_LIB)
        MESSAGE(FATAL_ERROR "SystemC-AMS library <systemc-ams> could not be found, pathes searched are ${SYSTEMCAMS_LIBRARY_SEARCH_PATH}, suffix is <lib-${SYSTEMCAMS_ARCH_DIR_SUFFIX}>\n--------->Try setting SYSTEMCAMS_HOME to the root of your SystemC installation")
      ENDIF(NOT SYSTEMCAMS_LIB)
    ENDIF(NOT SYSTEMCAMS_LIB)
    SET(SYSTEMCAMS_LIBRARIES  optimized ${SYSTEMCAMS_LIB} debug ${SYSTEMCAMS_LIB} CACHE FILEPATH "SYSTEMCAMS_LIBRARIES")
  ENDIF(MSVC)
ENDIF(SYSTEMCAMS_LIBRARIES)

INCLUDE_DIRECTORIES(${SYSTEMCAMS_INCLUDES})
MESSAGE(STATUS "SYSTEMCAMS_LIBRARIES: ${SYSTEMCAMS_LIBRARIES}")
# kate: space-indent on; indent-width 2; replace-tabs on;
