//-------------------------------------------------------------------------------------------
// Copyright (c) 2008-2010, Institute of Computer Technology, University of Technology Vienna
// Authors:
//   Christoph Grimm, Jan Haase, Markus Damm, Jiong Ou,  
//   Joseph Wenninger, Peter Brunmayr, Muhammad Farooq
// Contact:
//   tuv-ams-lib@ict.tuwien.ac.at
// For the license please read LICENSE.txt
//-------------------------------------------------------------------------------------------

#include "TUV_ams_lib_demangle.h"

// demangling currently supported for GCC >= 3.1.x
#if    ( __GNUC__ > 3 ) || ( ( __GNUC__ == 3 ) && ( __GNUC_MINOR__ > 0 ) )

#  define AHES_HAVE_CXA_DEMANGLE_
#  include <cstdlib>
#  include <cxxabi.h>

#endif // GCC >= 3.1

namespace TUV_ams_lib {

inline
std::string do_demangle( const char *mangled )
{
#ifdef AHES_HAVE_CXA_DEMANGLE_
  int status;
  char *tmp = __cxxabiv1::__cxa_demangle (mangled, 0, 0, &status);
  if( status<0 )
    return std::string("(Error while demangling name: ")
           + std::string(mangled)
           + std::string(")");

  std::string result(tmp);
  free(tmp);
  return result;
#else
  return std::string( mangled );
#endif
}

} // namespace TUV_ams_lib

#undef AHES_HAVE_CXA_DEMANGLE_

// kate: space-indent on; indent-width 2; replace-tabs on;
